import vue from '@vitejs/plugin-vue'
import AutoImport from 'unplugin-auto-import/vite'
import { NaiveUiResolver } from 'unplugin-vue-components/resolvers'
import Components from 'unplugin-vue-components/vite'
import { defineConfig } from 'vite'
import Uni from '@dcloudio/vite-plugin-uni'
import UniKuRoot from '@uni-ku/root'
import { resolve } from 'node:path'
import UnoCSS from 'unocss/vite'

export default defineConfig({
  envDir: './env', // 自定义env目录
  resolve: {
    alias: [
      {
        find: "@",
        replacement: resolve(process.cwd(), 'src')
      }
    ]
  },
  server: {
    host: '0.0.0.0', // 监听所有网络接口
    port: 2367,
    // 选项写法
    proxy: {
      '/pag': {
        target: 'https://cdn.tmui.design',
        changeOrigin: true,
        rewrite: (path) => path.replace(/^\/api/, '/api')
      },
    }
  },
  plugins: [
    Uni(),
    UniKuRoot(),
    UnoCSS(),
    AutoImport({
      imports: [
        'vue',
        {
          'naive-ui': [
            'useDialog',
            'useMessage',
            'useNotification',
            'useLoadingBar'
          ]
        }
      ]
    }),
    Components({
      resolvers: [NaiveUiResolver()]
    })
  ],
  css: {
    preprocessorOptions: {
      scss: {
        additionalData: `@import "@/static/css/color.scss";`,
      },
    },
  }
})
